.notebook {
    width: 80%;
    margin: 0 auto;
    font-family: Arial, sans-serif;
    overflow: auto;

}

.cell {
    padding: 20px;
    margin: 10px 0;
    border-radius: 5px;
}

.code {
    background-color: #272822;
    color: #f8f8f2;
    padding: 10px;
    overflow-x: auto;
}